' This program suits a repeater with gateway or a link (depending on address dec 55 in EEDATA)
' written in PROTON+ by John Drew VK5DJ - Version 3.2 2nd September 2004 - add change polarity of PTT

' Remote controls:  1= inhibit main TX	   2=inhibit link TX   	   3=timeout inhibit		
'  		 			4= require CTCSS-link  5= require CTCSS-rptr   6=change callsignf
'					7= change call delay   8= change timeout 	   9=flip link/gateway
'					   		  	   		   *=reset all controls

' The morse messages are stored in eeprom. A dit is HEX 4, a dah is HEX 0E, 
' a character space is HEX 50 and end of message is HEX 55. 
' Rules for values: dit and dah must be less than hex 1E, char gap must be more than hex 1E
' but NOT equal hex 55, the end of message MUST equal 55

' EEDATA address information follows in decimal:
' 0..41 	  callsign 
' 42,43,44 	  passwords
' 45		  mute direction (0= both active high, 1=rpt low & simp high, 2=rpt high & simpl low, 3=both active low)
' 46		  callsign delay 
' 47		  timeout delay
' 48..53	  timeout message
' 54		  ptt direction (0= both active high, 1=rpt low & simp high, 2=rpt high & simpl low, 3=both active low)
' 55		  0=link, 1=gateway  
' 56..63	  OK message 


Device 16F84
Config WDT_OFF,XT_OSC,PWRTE_ON,CP_OFF 				'set fuses 
'OPTIMISER_LEVEL=2
Dim Callsent  	 		 		 As Byte  	'To control a callsign after activity ceases
Dim CTCSSrcvd 					 As Byte	'To provide a short memory of a CTCSS input
Dim A 							 As Byte 	'loop counter variable
Dim Duration 					 As Byte	'duration of sound
Dim C_units 					 As Word	'callsign delay units loop counter
Dim C_tens 						 As Byte	'callsign delay tens counter
Dim T_units 					 As Word	'timeout delay units counter
Dim T_tens 						 As Byte	'timeout delay tens counter
Dim Tone1 						 As Byte	'Remote incoming address or data byte
Dim Tone2 						 As Byte	'This is the remote control byte
Dim Address 					 As Byte	'This is used to define address of morse in eeprom
Dim Digit1						 As Byte    'Used in remote updates of callsign and delays
Dim Digit2 		 		 		 As Byte    'Used in remote updates of callsign and delays
Dim Value						 As Byte	'Temp variable in remote control and tail
Dim Password 	   				 As Byte	'First password numeral
Dim RTXenabled 					 As Bit    	'If true will inhibit main TX
Dim STXenabled 					 As Bit  	'If true will inhibit link/gateway TX
Dim Remcontrol_inhib_timeout 	 As Bit 	'If true will inhibit timeout
Dim SCTCSSenabled 				 As Bit	 	'If true will enable CTCSS for access to simplex
Dim RCTCSSenabled 				 As Bit	 	'If true will enable CTCSS for access to rptr
Dim Rmuteactive 				 As Bit		'Indicates if input A (repeater) is active (1=active)
Dim Smuteactive 				 As Bit		'Indicates if input B (simplex) is active (1=active)
Dim muteactive 					 As Bit		'Indicates if one of the mutes was active
Dim Direction_A 				 As Bit		'Read from EEDATA at 2D (1=rptr mute active low, 0=rptr mute active low
Dim Direction_B 				 As Bit		'Read from EEDATA at 2D (1=link mute active low,0=link mute active high
Dim GateLink 					 As Bit		'Determines if gateway program 1 or link 0
Dim	StateR 						 As Bit		'Determines what is permissible for the Rptr TX
Dim	StateS 						 As Bit		'Determines what is permissible for the link/gateway
Dim PTThighR				 	 As Bit		'Holds phase of PTT repeater, 1=high, 0=low
Dim PTThighS				 	 As Bit		'Holds phase of PTT link, 1=high, 0=low
Dim TailF						 As Bit		'Tells rules if the call came from a tail
Dim Tailbeep					 As Bit		'Flags if a tail after CTCSS has opened
Dim Note						 As Byte	'Holds value of beep tone


		'Time out constants- each ten in EEDATA is approx 18 secs with 4mHz clock
Symbol Timeoutunits=2000	 	  	  	 	'Constant for timeout timer units counter -normally 2000
Symbol Calltimeunits=2000		  			'Constant for call timer units counter - normally 2000

	   	'miscellaneous compiler constants to be set by programmer
Symbol CTCSSvalue=150			  		    'The delay factor for CTCSS (anti jitter)
Symbol Start_call=0				  			'Start of callsign message in eedata
Symbol Password_add1=42			  			'Address in EEDATA for first password digit
Symbol Password_add2=43			  			'Address in EEDATA for first password digit
Symbol Password_add3=44			  			'Address in EEDATA for first password digit
Symbol Direction_add=45			  			'Address in EEDATA of direction value (Hex 2D)
Symbol Calltimeout_add=46		  		    'Address in EEDATA for calltimeout value (Hex 2E)
Symbol Timeout_add=47						'Address in EEDATA for timeout value (Hex 2F)
Symbol Start_to=48				  		    'Address in EEDATA for start of Timeout message
Symbol PTTdirection=54						'Address in EEDATA for PTT direction
Symbol Gateway=55				  		    'Address in EEDATA for gateway or link
Symbol Start_ok=56				  		    'Address in EEDATA for start of OK message
Symbol Highnote=118				  			'Sets freq of beep
Symbol Lownote=108				  			'Sets freq of boop

	   	'port symbols
Symbol Tone_out=PORTB.0			  			'Tone from sound generator
Symbol CTCSSin=PORTB.1			  			'CTCSS detect
Symbol TXrON=PORTB.3			  			'PTT of main TX (duplex rig)
Symbol TXsON=PORTB.2			  			'PTT of second TX (simplex rig)
Symbol Mute_A=PORTB.4			  			'Mute from duplex RX
Symbol Mute_B=PORTB.5			  			'Mute from simplex RX
Symbol Audio_A=PORTB.7			  			'Audio from duplex TX
Symbol Audio_B=PORTB.6			  			'Audio from simplex TX
Symbol Audio_OUT=PORTA.4		  			'Switches callsign audio
Symbol DTMF=PORTA				  			'Bring in the five bits of portA, AND %01111 
	   							  			'to remove A.4 
								  			'A.0-A.3 are tone decode, A.4= is call switch out
								  
	    'To establish the data in high memory of the PIC
Symbol	dit=$04		  	  	   	  	   		'Constant for a dit etc
Symbol  dah=$0E
Symbol	space=$50
Symbol  EOM=$55	 				  			'End of message

EData	dit,dit,dit,dah,space,dah,dit,dah,space,dit,dit,dit,dit,dit,space,dit,dah,dit,space
EData	dah,dah,dah,space,dit,dit,dit,dit,EOM						'End of callsign VK5ROH
EData	$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF			'Empty spaces for more callsign
EData   $55															'Emergency stop for callsign in case of bad entry
EData	$03,$09,$08													'Password 987
EData	$03,$42,$14													'Direction (both active low),Call delay, Timeout
EData	dah,space,dah,dah,dah,EOM									'Timeout message
EData	$03															'PTT direction (both active low)
EData	$01															'0=link, 1=gateway
EData	dah,dah,dah,space,dah,dit,dah,EOM							'OK message								  

Declare XTAL 4

TRISB=%00110010					  									'(outputs)	RB0, RB2, RB3, RB6, RB7
                        		  									'(inputs) 	RB1,RB4, RB5
TRISA=%00001111             	  									' Configure RA0 (PortA.0-3) as inputs
								  									' and A.4 as output


GoTo INITIALISE														'Jump over all the sub routines

TXs_OFF:  If PTThighS=1 Then Clear TXsON:Else:Set TXsON				'Turn off secondary TX
          If Rmuteactive=0 Then Clear Audio_A						'Turn repeater audio off only if repeater off air
          Return
		 
TXs_ON:   If StateS=0 Then Return	  								'TX is disabled (see rules)
		  If PTThighS=1 Then Set TXsON:Else:Clear TXsON				'Turn second TX on and provide audio
		  Return
		
TXr_OFF:  If PTThighR=1 Then Clear TXrON:Else:Set TXrON				'Turn off main TX
          Clear Audio_B												'audio to main TX from secondary receiver
		  Clear Audio_A												'audio from repeater receiver
          Return 	   

TXr_ON:  If StateR=0 Then Return									'TX is disabled for some reason (see rules)
		 If PTThighR=1 Then Set TXrON:Else:Clear TXrON			 	'Turn main TX on and provide audio
		 Tailbeep=1													'a TX has been on so allow a beep
		 Return

'This routine creates a delay loop of C_UNITS*C_TENS*DELAY(4) or whatever
'If delayMS=4 then loop gives about 40 minutes if C_TENS=255 and C_UNITS =2000
'Then prepares the input port variables - there is also a delay in timeout - changes made to prevent double count      
GETPORT: If C_tens>0 Then 		  	 			 				 	'Call timer has timed out
            If muteactive=0 Then DelayMS 4							'delay 4 only if delayms 4 not working in timeout
        	If C_units>0 Then Dec C_units							'then count down
			If C_units=0 Then 
			 			  Dec C_tens
        	 			  C_units=Calltimeunits
			 			  EndIf
         EndIf    

		 If CTCSSrcvd>0 Then Dec CTCSSrcvd 		 			 		'count it back to require long term trigger
		 If CTCSSin=0 Then CTCSSrcvd=CTCSSvalue	    				'check for CTCSS rcvd on Rptr (or gateway open)
		 
		 Rmuteactive=Mute_A^Direction_A    				 	 		'Direction read from EEDATA to determine +ve or -ve going port
         Smuteactive=Mute_B^Direction_B					 			'If direction is 1 then input is complement of mute
		 If Rmuteactive=1 Then Smuteactive=0    					'Input from repeater has priority
		 muteactive=Rmuteactive | Smuteactive		 		  		'muteactive in remote control- flags if both mutes off with 1
		 Return   		 
		 
CLEAR_REMOTES:			 		  									'Clear all the remote control bits
		 RTXenabled=0								   			 	'Repeater enabled
		 STXenabled=0												'Link/gateway enabled
		 Remcontrol_inhib_timeout=0									'Timeout enabled
		 SCTCSSenabled=1											'CTCSS cancelled on link/gateway
		 RCTCSSenabled=1											'CTCSS cancelled on repeater
		 Return

BOTH_OFF:GoSub TXs_OFF
		 GoSub TXr_OFF
		 Clear Audio_OUT
		 Return

'this routine is a listening routine during callsign playing to avoid duplicated code in MORSE
TAIL_SUPPORT:
		 GoSub GETPORT
		 GoSub RULES
		 GoSub TXr_ON
		 GoSub TXs_ON
		 Return	   			 	 

MORSE:   GoSub TAIL_SUPPORT
		 Value=0
		 Repeat			 											'allow mute to open audio if necessary,
		 	   DelayMS 5											'and a period before the call starts
			   GoSub TAIL_SUPPORT		   
			   Inc Value
		 Until Value=40
		 
		 Repeat
		 	   GoSub TAIL_SUPPORT
		       'Now send the call sign
		  	   Duration=ERead Address		  		  			 	'0(callsign), 48(Timeout) or 56(OK)
		  	   Inc Address
		  	   If Duration>29 Then
				 DelayMS Duration
			   Else 
			     Sound Tone_out,[118,Duration]
				 DelayMS 70
			   EndIf	  
	     Until Duration=85 	  		  				 	   			'85dec = $55			   
	     Return	 
		 
BEEPSND: Sound Tone_out,[Note,6]	  								'make a sound
		 DelayMS 150												'A short tail after the beeps
		 Return
		 
BEEP:	 Clear Audio_B												'Clear audio ports
		 Clear Audio_A
		 GoSub BEEPSND
		 Return

CALLS: 	 If C_tens<>0 Then Return   		  						'Call timer not yet expired
         Address=Start_call				  							'Set eeprom address for message
		 GoSub MORSE						  						'Send call
         C_units=Calltimeunits			  							'Set callsign delay variables
         C_tens=ERead Calltimeout_add
		 Callsent=1						  							'For testing later for tail
         Return

'This routine sends a "TO" then switches off the TX.
        
TO_CALL: GoSub TXr_ON
		 GoSub TXs_ON												'only if there is not TO on link
		 DelayMS 500						  	 	 				'delay 500msec
		 Address=Start_to
		 Repeat
	   		 Duration=ERead Address	  	  							'0(callsign), 48(Timeout) or 56(OK)
		  	 Inc Address
		  	 If Duration>29 Then 
			   DelayMS Duration 
			 Else 
			   Sound Tone_out,[118,Duration]
			   DelayMS 70 
			 EndIf  
	   	 Until Duration=85
         GoSub BOTH_OFF					  		   			  		'Turn them all off just in case
         Return       

'This routine controls the sending of the time out message
TIMEOUT: If Remcontrol_inhib_timeout=1 Then 
		 							   T_tens=255  	  	    		'remote cancel timer (just large number)
		 							   Return
		 							   EndIf
		 Clear Audio_A	   				  	 						'ensure that both rx audios are off
		 Clear Audio_B
		 Set Audio_OUT
		 GoSub TO_CALL			 	      							'send "to"
LOOP_T:	 GoSub GETPORT					  							'check to see if mute still open
		 If muteactive=1 Then GoTo LOOP_T	  						'if so then wait
		 Set Audio_OUT
		 If STXenabled=0 Then StateS=1:Else:StateS=0	  	 	  	'allow link or gateway to send TO
		 GoSub TO_CALL					  							'send "to" again so hear time out
         Return
		 	
'these 2 routines manage incoming DTMF - normally wait4notone flows into wait4tone 		
WAIT4NOTONE:															
		 While DTMF&15>0    								 		'now wait until the tone goes off
		 GoSub GETPORT										 		'check the port to see if changed
		 If muteactive<>1 Then Return							 	'if both mutes shut then abandon
		 Wend			  	   										'otherwise fall through to wait4tone
WAIT4TONE:			
		 While DTMF&15=0		 							 		'now wait until there is a tone again
		 GoSub GETPORT										 		'check the port to see if changed
		 If muteactive<>1 Then Return							 	'if both mutes shut then abandon
		 Wend	  		  	   										'otherwise return with a byte
		 Return

GETREMOTEDATA:
		 GoSub WAIT4TONE											'waits and brings in next byte
		 If muteactive<>1 Then Return							 	'if both mutes shut then abandon
		 Digit1=DTMF&15 			 						 		'assign value to high byte
			 
		 GoSub WAIT4NOTONE											'waits and brings in next byte
		 If muteactive<>1 Then Return							 	'if both mutes shut then abandon
		 Digit2=DTMF&15 			 						 		'assign value to low byte
		 
		 While DTMF&15>0    								 		'now wait until the tone goes off
		 GoSub GETPORT										 		'check the port to see if changed
		 If muteactive<>1 Then Return							 	'if both mutes shut then abandon
		 Wend
		 If Digit1=10 Then Digit1=0							 		'a zero comes through as a 10
		 If Digit2=10 Then Digit2=0	
		 Value=16*digit1+digit2								 		'calculate value of numbers input
		 Return
		 
CHANGE_CALL:
			Address=0	 									 		'set address at start of callsign area in eeprom
			Repeat
				  GoSub GETREMOTEDATA
				  GoSub GETPORT									 	'check the port to see if changed
				  If muteactive<>1 Then Return						'if mute has dropped then abandon
				  If Value=255 Then Return						 	'a ff also causes the system to drop (used as loop).
				  If Address=password_add1-1 Then Return		 	'make sure ident doesn't exceed memory allocated
				  EWrite Address, [Value] 						 	'write the hex byte to eedata 
				  Inc Address			  
			Until Value=255				
			Return						 	  				 		'a hex 55 is at calltimeout_add-1 to ensure there is an end of call 
			
CHANGE_CALL_DELAY:
			GoSub GETREMOTEDATA
			GoSub GETPORT						  	  		 		'check the port to see if changed
			If muteactive<>1 Then Return
			EWrite Calltimeout_add, [Value] 
			Return
			
CHANGE_TIMEOUT:
			GoSub GETREMOTEDATA
			GoSub GETPORT							  		 		'check the port to see if changed
			If muteactive<>1 Then Return
			If Value<4 Then Value=4			  	   	  		 		'if timeout too short cannot remote
			EWrite Timeout_add, [Value] 
			Return 
			
CHANGE_LINK:
			Value=ERead Gateway
			If Value=1 Then EWrite Gateway, [0]: Else: EWrite Gateway, [1]
			Return	 		   			   					 		'gateway=1  link=0
		 
CONTROLTONE: Clear Audio_A					 		  		 		'turn off audios during tone in
			 Clear Audio_B
			 GoSub GETPORT
			 If muteactive<>1 Then Return		  		  		 	'if both mutes shut then abandon loop
			 
			 Tone1=DTMF&15						  	  		 		'assign value of port to address variable
			 Password=ERead Password_add1			  		 		'get first password numeral
			 If Tone1=10 Then Tone1=0                               'fix key
			 If Tone1<>Password Then GoTo CONTROLTONE 		 		'first digit of password
			 
			 GoSub WAIT4NOTONE										'waits and brings in next byte
		     If muteactive<>1 Then Return							'if both mutes shut then abandon
			 Tone1=DTMF&15 			 				  		 		'assign value to second tone
			 If Tone1=10 Then Tone1=0
			 Password=ERead Password_add2			  		 		'get second password numeral
			 If Tone1<>Password Then GoTo CONTROLTONE 		 		'second digit of password
			 
			 GoSub WAIT4NOTONE										'waits and brings in next byte
		 	 If muteactive<>1 Then Return							'if both mutes shut then abandon
			 Tone1=DTMF&15 			 				  		 		'assign value to third tone
			 If Tone1=10 Then Tone1=0
			 Password=ERead Password_add3			  		 		'get third password numeral
			 If Tone1<>Password Then GoTo CONTROLTONE 		 		'third digit of password
			 
			 GoSub WAIT4NOTONE										'waits and brings in next byte
		 	 If muteactive<>1 Then Return							'if both mutes shut then abandon
			 Tone2=DTMF&15 			 				  		 		'assign value to the control byte in tone2
			 While DTMF&15>0						  		 		'now wait until the tone goes off
			 	   GoSub GETPORT					  		 		'check the port to see if changed
				   If muteactive<>1 Then Return		  		 		'if both mutes shut then abandon
			 Wend 
			 'now assign to the remotecontrol variable
			 If Tone2=1 Then 
			 			RTXenabled=1   		  		 				'if true will inhibit main tx 
						STXenabled=1								'inhibit link too
						EndIf
			 If Tone2=2 Then STXenabled=1			 		 		'if true will inhibit link
			 If Tone2=3 Then Remcontrol_inhib_timeout=1		 		'if true will inhibit timeout
			 If Tone2=4 Then SCTCSSenabled=0		   		 		'changed for Santiago for CTCSS as default-if true will enable ctcss for simplex access
			 If Tone2=5 Then RCTCSSenabled=0		   		 		'changed for Santiago for CTCSS as default-if true will enable ctcss for rptr access
			 If Tone2=6 Then GoSub CHANGE_CALL						'enter a new call
			 If Tone2=7 Then GoSub CHANGE_CALL_DELAY				'enter callsign delay
			 If Tone2=8 Then GoSub CHANGE_TIMEOUT					'enter main timer delay
			 If Tone2=9 Then GoSub CHANGE_LINK 						'alternate gateway and link
			 If Tone2=11 Then GoSub CLEAR_REMOTES 	   		 		'* to reset
			 While muteactive=1 						   		 	'now wait until both mutes high then send ok.
			 	   GoSub GETPORT					   		 		'check the port to see if changed	   
			 Wend	 			  	   
			 	 'now send an ok
			 GoSub BOTH_OFF							   		 		'get all off
			 TailF=1												'set tail indicator (used in RULES)
			 GoSub RULES					   		 	 			'prepare for ok
			 GoSub TAIL_SUPPORT
			 DelayMS 500							   		 		'1/2 sec tail on repeater only						
        	 Address=Start_ok						   		 		'start of ok message in eeprom
			 GoSub MORSE							   		 		'send message
			 GoSub BOTH_OFF							   		 		'everything off
			 C_tens=ERead Calltimeout_add							'so no call is sent after controlling
			 TailF=0												'cancel tail indicator
			 Return									   		 		'end of remote control sequence	

'This routine determines the tail features
		 
TAIL:	 	 If Tailbeep=1 Or C_tens=0 Then
			 TailF=1		 	 	  		  						'flag to indicate this is a tail to rules
			 GoSub TAIL_SUPPORT  
		 	 Set Audio_OUT 			  								'tone on
		 	 DelayMS 150					 						'Provide a tail
		 	 GoSub CALLS			 	  	   						'Generate call if due
		 	 If Remcontrol_inhib_timeout=1 And Callsent=0 Then 		'If timer inhibited send 2 beeps if call not due
		   		   	   	   			       			  GoSub BEEP	'Prepare for beep and send it
									    			  GoSub BEEPSND 'Send the second beep
		 	 EndIf
		 	 If STXenabled=0 And Callsent=0 Then GoSub BEEP  	 	'If gateway on send a beep
			 Clear Audio_OUT
			 TailF=0	
			 EndIf	 	 				 	   				'reset tail flag
		 	 Return   			
			
'this routine sets the rules for transmitting and shuts the transmitters
'the turning on of the TXs happens in the routines that use RULES

RULES:	If Rmuteactive=1 Then	 			 				 		'if repeater mute open then......
			Clear Audio_B
			If RTXenabled=1 Then StateR=0: Return 	 	 			'if not OK to TX on rptr then abandon
			If RCTCSSenabled=1 And CTCSSrcvd>0 Or RCTCSSenabled=0 Then
				StateR=1		  	 			 			 		'advise txR_on that it's OK to TX
				Set Audio_A
				If STXenabled=1 Then StateS=0:Return:Else:StateS=1	'advise txS_on that it's not OK to TX
			Else
				StateR=0			 								'inhibit rptr
				StateS=0			 								'inhibit simplex
				Clear Audio_A
				GoSub BOTH_OFF
			EndIf	
		Else		 'matches if Rmuteactive=1	
			If Smuteactive=1 Then
		   	   Clear Audio_A
			   If RTXenabled=1 Or STXenabled=1 Then 				'if either disabled inhibit both	
			   	  			   	  			   StateR=0 
											   GoSub BOTH_OFF
											   Return 
			   EndIf  'matches if RTXenabled=1 or STXenabled=1		 		 
			   If SCTCSSenabled=1 And CTCSSrcvd>0 Or SCTCSSenabled=0 Then 
			   	  StateR=1		  	  			  	 				 'its OK for simplex to turn rptr on
				  Set Audio_B
				  Return
			   Else 
			   	  StateR=0	 										 'shut rptr down
				  GoSub BOTH_OFF
				  Return
			   EndIf 'matches if SCTSSenabled=1 or ...	  
			Else     'matches if Smuteactive=1
			   'if it gets this far no mute is active
			   If TailF=1 Then	   	  			  	 				 'its a tail so do the following
			   	  Clear Audio_A										 'shut down receiver audios 
			   	  Clear Audio_B
			   	  If RTXenabled=0 Then								 'rptr not disabled
			   	  	 StateR=1	  									 'so approve rptr tx
					 Set Audio_OUT									 'put CW audio on
				  	 If STXenabled=0 And GateLink=1 Then 			 'if simplex approved and its a gateway
					 	StateS=1	 	 							 'then approve simplex tx
					 	Return
				  	 Else
				  	 	StateS=0	 	 				 			 'disable link
						GoSub TXs_OFF	 				 			 'turn link off
				     EndIf
				  Else	  'matches if RTXenabled
					 StateR=0 										 'disable Rptr if Rptr disabled
			      EndIf   'matches if RTXenabled
			   EndIf	  'matches if tailF=1
			EndIf	  	  'matches if Smuteactive=1
		EndIf			  'matches if Rmuteactive=1
		Return	  

TIMECOUNT:	
		If T_tens=0 Then 
	   			GoSub TIMEOUT	  		  					 		'Go to timeout - send "TO" and shut TX
	   			Return					  					 		'then return without changing time count
				EndIf
		DelayMS 4							  					 	'This is the basic timer multiplied by loop
    	If T_units>0 Then Dec T_units		  					 	'two decremented loops
    	If T_units=0 Then 
	   			 Dec T_tens
       			 T_units=Timeoutunits
		EndIf
        Return
		
INITIALISE: 
		DelayMS 1000
		T_units=Timeoutunits  		  					 			'Initialise the call timer and timeout timer
		T_tens=ERead Timeout_add	  					 			'Get timeout value from EEDATA
        C_units=Calltimeunits
        C_tens=ERead Calltimeout_add  					 			'Get call timeout from EEDATA at hex 2E
		
		Digit1=ERead Direction_add 	  					 			'Get mute direction info from EEDATA
		Direction_A=GetBit Digit1,0	  					 			'Gets the value of the 0 bit
		Direction_B=GetBit Digit1,1   					 			'Gets the value of the 1 bit
		
		Digit1=ERead Gateway							 			'Reads if gateway or link
		GateLink=GetBit Digit1,0						 			'Set the gatelink flag
		
		Digit1=ERead PTTdirection									'Bring in PTTdirection
		PTThighR=GetBit Digit1,0									'Assign direction to rptr PTT
		PTThighS=GetBit Digit1,1									'Assign direction to rptr PTT
		
		Callsent=0					  					 			'No recent call sign sent
        GoSub BOTH_OFF				  					 			'Ensure everything is initialised off
		GoSub CLEAR_REMOTES			  					 			'Ensure all remote control functions are zeroed
		StateR=0										 			'Establish not OK to TX in rptr
		StateS=0										 			'Establish not OK to TX in simplex
	
MAIN:   GoSub GETPORT										 		'At least one pin low so TX
		If muteactive=0 Then 	  		  	  		  	  		    'Now clean up and send a call when it times out
		   	If Callsent=0 And C_tens=0 Then GoSub TAIL				'Send a tail or a call if no activity	
			If muteactive=0 Then GoSub BOTH_OFF	  					'while at CALL (from tail) the mutes can open
			GoTo MAIN
		EndIf
		If Rmuteactive=1 Then Note=Highnote: Else: Note=Lownote		'Determine frequency of beep
		
LOOP:	GoSub GETPORT	 	  				 	  					'This is the main loop when a mute is open
		While muteactive=1
 	      GoSub RULES
		  GoSub TXr_ON
		  GoSub TXs_ON
		  If DTMF&%01111>0 Then GoSub CONTROLTONE
	      GoSub TIMECOUNT 
		  GoTo LOOP
		Wend
        'once in this next section the mutes are shut. First reset the timeout timer
	    T_units=Timeoutunits   								   		'Set TX timers according to EDATA
        T_tens=ERead Timeout_add
	   
	   'This section is the antiflutter mechanism. Once repeater mute shuts these lines listen 400ms
	   'In effect this creates the main tail
	   If GateLink=0 Then GoSub TXs_OFF			   	  		   		'if link no tail
	   Clear Audio_B									 	   		'silence the audio
	   Clear Audio_A
	   Value=80										 	   		    'set a counter for 80 loops of 5ms
	   Repeat
	   		 GoSub GETPORT								 	   		'Check the port
	   		 If muteactive=1 Then GoTo MAIN				 	   		'if  a mute opens then go again (flutter?)
	   		 DelayMS 5  			 					 	   		'10 ms delay
	   		 Dec Value
	   Until Value=0		
	   'end of anti flutter
		
	   Callsent=0											   		'Reset call indicator
	   GoSub TAIL											   		'Now send a tail or a callsign
		
	   CTCSSrcvd=0											   		'Reset CTCSS
	   If muteactive=0 Then GoSub BOTH_OFF							'Make sure both TXs off (can pick up in CALL)			  							   'Turn both TXs off and all audios
       StateR=0	 										   			'not OK to TX
	   StateS=0
	   Tailbeep=0
       GoTo MAIN							    			   		'loop to restart cycle
End

